#322 "general.ltx"
let try_finalize f x finally y =
  let res = try f x with exn -> finally y; raise exn in 
  finally y; 
  res
#523 "fich.ltx"
open Sys;;
open Unix;;

let iter_dir f dirname =
  let d = opendir dirname in
  try while true do f (readdir d) done
  with End_of_file -> closedir d
#381 "sign.ltx"
let rec restart_on_EINTR f x = 
  try f x with Unix_error (EINTR, _, _) -> restart_on_EINTR f x
#401 "sign.ltx"
let free_children signal = 
  try while fst (waitpid [ WNOHANG ] (-1)) > 0 do () done 
  with Unix_error (ECHILD, _, _) -> ()
#463 "sign.ltx"
open Sys;;
open Unix;;

let exec_as_system exec args = 
  let old_mask = sigprocmask SIG_BLOCK [sigchld ] in 
  let old_int = signal sigint Signal_ignore in 
  let old_quit = signal sigquit Signal_ignore in 
  let reset() =
    ignore (signal sigint old_int); 
    ignore (signal sigquit old_quit);
    ignore (sigprocmask SIG_SETMASK old_mask) in
  let system_call () = 
    match fork() with
    | 0 -> 
        reset();
        begin try 
          exec args 
        with _ -> exit 127 
        end
    | k -> 
        snd (restart_on_EINTR (waitpid []) k) in
  try_finalize system_call() reset();;

let system cmd = 
  exec_as_system (execv "/bin/sh") [| "/bin/sh"; "-c"; cmd |];;
#1361 "comm1.ltx"
let rec really_write fd buffer offset len =
  let n = restart_on_EINTR (single_write fd buffer offset) len in
  if n < len then really_write fd buffer (offset + n) (len - n);;
#412 "comm2.ltx"
let retransmit fdin fdout =
  let buffer_size = 4096 in
  let buffer = String.create buffer_size in
  let rec copy() = 
    match read fdin buffer 0 buffer_size with
      0 -> ()
    | n -> ignore (write fdout buffer 0 n); copy() in
  copy ();;
#564 "comm2.ltx"
let install_tcp_server_socket addr = 
  let s = socket PF_INET SOCK_STREAM 0 in
  try  
    bind s addr; 
    listen s 10; 
    s
  with z -> close s; raise z;;
#583 "comm2.ltx"
let tcp_server treat_connection addr = 
  ignore (signal sigpipe Signal_ignore);
  let server_sock = install_tcp_server_socket addr in 
  while true do
      let client = restart_on_EINTR accept server_sock in
      treat_connection server_sock client
  done;;
#639 "comm2.ltx"
let sequential_treatment server service client = service client 
#666 "comm2.ltx"
let fork_treatment server service (client_descr, _ as client) =  
  let treat () = 
    match fork() with
    | 0 -> close server; service client; exit 0
    | k -> () in
  try_finalize treat () close client_descr;;
#702 "comm2.ltx"
let double_fork_treatment server service (client_descr, _ as client) = 
  let treat () =
    match fork() with
    | 0 -> 
        if fork() <> 0 then exit 0;
        close server; service client; exit 0
    | k ->  
        ignore (restart_on_EINTR (waitpid []) k) in
  try_finalize treat () close client_descr;;
